"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/promise"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _toolLib = require("@kettil/tool-lib");

var _chalk = require("chalk");

var _licenseCompatibilities = _interopRequireDefault(require("../../../configs/licenseCompatibilities"));

var _licensePackages = _interopRequireDefault(require("../../../configs/licensePackages"));

var _config = _interopRequireDefault(require("../../cli/config"));

var _licenseDisabledError = _interopRequireDefault(require("../../errors/licenseDisabledError"));

var _licenseIncompatibleError = _interopRequireDefault(require("../../errors/licenseIncompatibleError"));

var _getPackageInfo = _interopRequireDefault(require("./helpers/getPackageInfo"));

var _getPackagePaths = _interopRequireDefault(require("./helpers/getPackagePaths"));

var _isLicenseCompatible = _interopRequireDefault(require("./helpers/isLicenseCompatible"));

const licensecheck = async argv => {
  var _context, _context2, _context3;

  const {
    cwd,
    log
  } = argv;
  const projectInfo = await (0, _getPackageInfo.default)(argv, {
    packagePath: cwd,
    ignorePackages: [],
    licensePackages: {}
  });

  if (!projectInfo) {
    throw new Error('Project package data was not found');
  }

  const projectLicense = projectInfo.license;
  const supportedLicenses = (0, _sort.default)(_context = (0, _keys.default)(_licenseCompatibilities.default)).call(_context);
  const compatibleLicences = _licenseCompatibilities.default[projectLicense];

  if (!(0, _toolLib.isArray)(compatibleLicences)) {
    throw new _licenseDisabledError.default('The project license has no compatibility group', `License verification is disabled (${projectLicense} != ${supportedLicenses.join('|')})`);
  }

  const configIgnorePackages = await (0, _config.default)(cwd, 'license.ignore.packages');
  const ignorePackages = (0, _toolLib.isArray)(configIgnorePackages) ? (0, _filter.default)(configIgnorePackages).call(configIgnorePackages, v => typeof v === 'string') : [];
  log.info('Get the paths of the packages');
  const packagePaths = await (0, _getPackagePaths.default)(cwd);
  log.info('Read the license data');
  const packageInfos = await _promise.default.all((0, _map.default)(packagePaths).call(packagePaths, packagePath => (0, _getPackageInfo.default)(argv, {
    packagePath,
    ignorePackages,
    licensePackages: _licensePackages.default
  })));
  log.info('Validation of the license data');
  const licenseCompatibleProps = {
    compatibleLicences: (0, _map.default)(compatibleLicences).call(compatibleLicences, v => v.toUpperCase()),
    projectLicense
  };
  const incompatibleLicenses = (0, _map.default)(_context2 = (0, _filter.default)(_context3 = (0, _filter.default)(packageInfos).call(packageInfos, v => v !== undefined)).call(_context3, packageInfo => !(0, _isLicenseCompatible.default)({ ...licenseCompatibleProps,
    packageInfo
  }))).call(_context2, ({
    name,
    license,
    version,
    path
  }, index) => [`${index + 1}`, name, license, version, path]);

  if (incompatibleLicenses.length > 0) {
    incompatibleLicenses.unshift([(0, _chalk.red)('#'), (0, _chalk.red)('Name'), (0, _chalk.red)('License'), (0, _chalk.red)('Version'), (0, _chalk.red)('Path')]);
    throw new _licenseIncompatibleError.default('Incompatible licenses found', incompatibleLicenses);
  }
};

var _default = licensecheck;
exports.default = _default;