"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.handler = exports.builder = exports.desc = exports.command = void 0;

var _path = require("path");

var _eslint = _interopRequireDefault(require("../actions/tools/eslint"));

var _prettier = _interopRequireWildcard(require("../actions/tools/prettier"));

var _tsc = _interopRequireDefault(require("../actions/tools/tsc"));

var _spinner = require("../cli/spinner");

var _yargs = require("../cli/yargs");

var _access = _interopRequireDefault(require("../cmd/access"));

const command = 'lint';
exports.command = command;
const desc = 'Run the code quality tools';
exports.desc = desc;
const services = ['eslint', 'prettier', 'typescript'];
const options = {
  group: `${command}-Options`
};

const isSelectedService = (value, service) => typeof value === 'undefined' || value === service;

const builder = (0, _yargs.builderDefault)(command, yargs => yargs.options({
  ci: { ...options,
    type: 'boolean',
    default: (0, _yargs.ciDefaultValue)(),
    describe: 'Run it in CI mode'
  },
  only: { ...options,
    choices: services,
    describe: 'Run a single code quality tool'
  }
}));
exports.builder = builder;

const handler = async argv => {
  try {
    await (0, _yargs.commonHandler)(argv, !argv.ci);
    const hasTypescriptConfig = await (0, _access.default)((0, _path.join)(argv.cwd, 'tsconfig.json'));

    if (isSelectedService(argv.only, 'prettier')) {
      await (0, _spinner.spinnerAction)((0, _prettier.default)(argv, argv.ci ? {
        write: false,
        extension: _prettier.extensionCi
      } : {
        write: true
      }), 'Prettier');
    }

    if (isSelectedService(argv.only, 'eslint')) {
      await (0, _spinner.spinnerAction)((0, _eslint.default)(argv, argv.ci ? {
        write: false
      } : {
        write: true
      }), 'ESLint');
    }

    if (isSelectedService(argv.only, 'typescript')) {
      if (hasTypescriptConfig) {
        await (0, _spinner.spinnerAction)((0, _tsc.default)(argv, {
          mode: 'type-check'
        }), 'Typescript');
      } else {
        argv.log.info('Typing check is skipped (tsconfig.json was not found) ');
      }
    }
  } catch (error) {
    (0, _yargs.errorHandler)(argv, error);
  }
};

exports.handler = handler;